package com.mt.wms.empty.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.CodeGeneratorHelper;
import com.mt.wms.core.utils.HttpClient;
import com.mt.wms.core.vo.R;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: liguanghao
 * @Date: 2021/12/13 19:54
 * @Version 1.0
 */
public class ScheduledTask extends BaseService {

    private final static Logger logger = LoggerFactory.getLogger(AsynRunTaskService.class);

    @Resource
    private KilnInfoServiceBiz kilnInfoServiceBiz;
    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private TaskDistanceUtils taskDistanceUtils;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private AsynRunTaskService asynRunTaskService;
    @Resource
    private PlcNameSpaceServiceBiz plcNameSpaceServiceBiz;
    @Resource
    private AlarmInfoServiceBiz alarmInfoServiceBiz;
    @Resource
    private ParRotSpeedValueServiceBiz parRotSpeedValueServiceBiz;
    @Resource
    private ParGasValueServiceBiz parGasValueServiceBiz;
    @Resource
    private ParTemValueServiceBiz parTemValueServiceBiz;
    @Resource
    private ParEleValueServiceBiz parEleValueServiceBiz;
    @Resource
    private PointInfoServiceBiz pointInfoServiceBiz;
    @Resource
    private RunTaskUtils runTaskUtils;

    /**
     * 查询加工完成的窑炉
     */
    @Scheduled
    public void taskForStockToKiln() throws IOException {
        //两辆车是否都空闲，只有两辆车都空闲才被允许做任务
        List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>().eq(VehicleInfo.STATUS, 0));
        if (vehicleInfoList.size()==2){
            //遍历currTask为进行中且is_in为正在加工的窑炉状态，若加工完毕，调用runTaskForOutKiln
            //  2022/3/6 有两托的判定哪一托好了(通过开始加工时间顺序排列)
            List<CurrTask> currTaskList = currTaskServiceBiz.list(new QueryWrapper<CurrTask>()
                    .eq(CurrTask.STATUS, 1)
                    .eq(CurrTask.IS_IN, 1)
                    .orderByAsc(CurrTask.BEGIN_TIME));
            for (CurrTask currTask :currTaskList
                    ) {
                Long kilnId = currTask.getKilnId();
                KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
                PlcNameSpace plcNameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                        .eq(PlcNameSpace.EQ_TYPE, 1)
                        .eq(PlcNameSpace.TYPE, 0)
                        .eq(PlcNameSpace.EQ_ID,kilnId));
                //通过允许出炉信号来判定是否加工完成
                String result = readPlcToString(plcNameSpace.getName(), "AllowOut");
                Boolean resultBoolean = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
                if (resultBoolean){
                    //调用RunTaskUtils.runTaskForOutKiln
                    runTaskUtils.runTaskForOutKiln(kilnId);
                    break;
                }
            }
        }
    }


    //查询出正在加工的窑炉，遍历，根据采集频率的不同，采集高频数据能源消耗。
    public void getFastValue(){
        //加工炉列表
        List<KilnInfo> kilnInfoList = kilnInfoServiceBiz.list(new QueryWrapper<KilnInfo>().in(KilnInfo.TYPE,1,3));
        for (KilnInfo kilnInfo:kilnInfoList
             ) {
            PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                    .eq(PlcNameSpace.EQ_TYPE, 1)
                    .eq(PlcNameSpace.EQ_ID, kilnInfo.getId())
                    .eq(PlcNameSpace.TYPE,1));
            String nameSpaceNote = nameSpace.getName();
            String result = readPlcToString(nameSpaceNote, "Working");
            Boolean resultBoolean = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
            //该炉子在工作中
            if (resultBoolean){
                Long kilnId = kilnInfo.getId();
                String identificationNumber = readPlcToString(nameSpaceNote, "Identification number");
                Long currTaskId = Long.valueOf(JSONObject.parseObject(identificationNumber).get("result").toString());
                //加工炉
                if (kilnInfo.getType()==1){
                    //一号油搅拌转速
                    Double oilStirringSpeed1 = readPlc(nameSpaceNote, "OilStirringSpeed1");
                    //二号油搅拌转速
                    Double oilStirringSpeed2 = readPlc(nameSpaceNote, "OilStirringSpeed2");
                    ParRotSpeedValue parRotSpeedValue=new ParRotSpeedValue();
                    parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed1.floatValue());
                    parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed2.floatValue());
                    parRotSpeedValue.setKilnId(kilnId);
                    parRotSpeedValue.setTaskId(currTaskId);
                    setCommonField(parRotSpeedValue);
                    parRotSpeedValueServiceBiz.save(parRotSpeedValue);
                    //氮气
                    Double nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                    //甲烷
                    Double methanolFlow = readPlc(nameSpaceNote, "MethanolFlow");
                    //丙烷
                    Double propaneFlow = readPlc(nameSpaceNote, "PropaneFlow");
                    //氨气
                    Double ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                    //实际碳势
                    Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                    //设定碳势
                    Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                    ParGasValue parGasValue=new ParGasValue();
                    parGasValue.setNitFlowValue(nitrogenFlow.floatValue());
                    parGasValue.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                    parGasValue.setMethanolFlow(methanolFlow.floatValue());
                    parGasValue.setPropaneFlow(propaneFlow.floatValue());
                    parGasValue.setActualNitPotValue(actualCarbon.floatValue());
                    parGasValue.setSetNitPotValue(setupCarbon.floatValue());
                    parGasValue.setKilnId(kilnId);
                    parGasValue.setTaskId(currTaskId);
                    setCommonField(parGasValue);
                    parGasValueServiceBiz.save(parGasValue);
                }
                //氮化炉
                if (kilnInfo.getType()==3){
                    //实际氮势
                    Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                    //设定氮势
                    Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                    //氢含量
                    Double hydrogenContent = readPlc(nameSpaceNote, "HydrogenContent");
                    //分解率
                    Double decompositionRate = readPlc(nameSpaceNote, "DecompositionRate");
                    //氨气量
                    Double ammoniaFlow=readPlc(nameSpaceNote,"AmmoniaFlow");
                    //氮气量
                    Double nitrogenFlow=readPlc(nameSpaceNote,"NitrogenFlow");
                    //二氧化碳量
                    Double carbonDioxideFlow = readPlc(nameSpaceNote, "CarbonDioxideFlow");
                    ParGasValue parGasValue3=new ParGasValue();
                    parGasValue3.setNitFlowValue(nitrogenFlow.floatValue());
                    parGasValue3.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                    parGasValue3.setActualNitPotValue(actualCarbon.floatValue());
                    parGasValue3.setSetNitPotValue(setupCarbon.floatValue());
                    parGasValue3.setHydrogenContent(hydrogenContent.floatValue());
                    parGasValue3.setDecompositionRate(decompositionRate.floatValue());
                    parGasValue3.setKilnId(kilnId);
                    parGasValue3.setTaskId(currTaskId);
                    setCommonField(parGasValue3);
                    parGasValueServiceBiz.save(parGasValue3);
                }
            }
        }
    }
    //查询出正在加工的窑炉，遍历，根据采集频率的不同，采集低频数据能源消耗。
    public void getSlowValue(){
        List<KilnInfo> kilnInfoList = kilnInfoServiceBiz.list();
        for (KilnInfo kilnInfo:kilnInfoList
             ) {
            PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                    .eq(PlcNameSpace.EQ_TYPE, 1)
                    .eq(PlcNameSpace.EQ_ID, kilnInfo.getId())
                    .eq(PlcNameSpace.TYPE,1));
            String nameSpaceNote = nameSpace.getName();
            String result = readPlcToString(nameSpaceNote, "Working");
            Boolean resultBoolean = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
            if (resultBoolean){
                Long kilnId = kilnInfo.getId();
                String identificationNumber = readPlcToString(nameSpaceNote, "Identification number");
                Long currTaskId = Long.valueOf(JSONObject.parseObject(identificationNumber).get("result").toString());
                //加工炉
                if (kilnInfo.getType()==1){
                    //实际温度
                    Double actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                    //设定温度
                    Double setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                    //油槽实际温度
                    Double actualOilTemp = readPlc(nameSpaceNote, "ActualOilTemp");
                    //油槽设定温度
                    Double setUpOilTemp = readPlc(nameSpaceNote,"SetUpOilTemp");
                    ParTemValue parTemValue=new ParTemValue();
                    parTemValue.setSetTemValue(setUpTemp.floatValue());
                    parTemValue.setActTemValue(actualTemp.floatValue());
                    parTemValue.setOilTankSetTemValue(setUpOilTemp.floatValue());
                    parTemValue.setOilTankActTemValue(actualOilTemp.floatValue());
                    parTemValue.setKilnId(kilnId);
                    parTemValue.setTaskId(currTaskId);
                    setCommonField(parTemValue);
                    parTemValueServiceBiz.save(parTemValue);
                    //电度值
                    Double electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                    //A向电压
                    Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                    Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                    Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                    ParEleValue parEleValue=new ParEleValue();
                    parEleValue.setAVoltagevValue(phaseAVoltage.floatValue());
                    parEleValue.setAVoltagevValue(phaseBVoltage.floatValue());
                    parEleValue.setAVoltagevValue(phaseCVoltage.floatValue());
                    parEleValue.setTotalBat(electricalValue.floatValue());
                    parEleValue.setKilnId(kilnId);
                    parEleValue.setTaskId(currTaskId);
                    setCommonField(parEleValue);
                    parEleValueServiceBiz.save(parEleValue);
                }
                //回火炉
                if (kilnInfo.getType()==2){
                    //实际温度
                    Double actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                    //设定温度
                    Double setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                    ParTemValue parTemValue2=new ParTemValue();
                    parTemValue2.setSetTemValue(setUpTemp.floatValue());
                    parTemValue2.setActTemValue(actualTemp.floatValue());
                    parTemValue2.setKilnId(kilnId);
                    parTemValue2.setTaskId(currTaskId);
                    setCommonField(parTemValue2);
                    parTemValueServiceBiz.save(parTemValue2);
                    //电度值
                    Double electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                    //A向电压
                    Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                    Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                    Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                    ParEleValue parEleValue2=new ParEleValue();
                    parEleValue2.setAVoltagevValue(phaseAVoltage.floatValue());
                    parEleValue2.setAVoltagevValue(phaseBVoltage.floatValue());
                    parEleValue2.setAVoltagevValue(phaseCVoltage.floatValue());
                    parEleValue2.setTotalBat(electricalValue.floatValue());
                    parEleValue2.setKilnId(kilnId);
                    parEleValue2.setTaskId(currTaskId);
                    setCommonField(parEleValue2);
                    parEleValueServiceBiz.save(parEleValue2);
                }
                //氮化炉
                if (kilnInfo.getType()==3){
                    //实际温度
                    Double actualTemp =readPlc(nameSpaceNote,"ActualTemp");
                    //设定温度
                    Double setUpTemp = readPlc(nameSpaceNote,"SetUpTemp");
                    //炉压
                    Double furnacePressure = readPlc(nameSpaceNote, "FurnacePressure");
                    //外一区温度
                    Double outerZoneITemperature = readPlc(nameSpaceNote, "OuterZoneITemperature");
                    //外二区温度
                    Double outerZone2Temperature = readPlc(nameSpaceNote, "OuterZone2Temperature");
                    ParTemValue parTemValue3=new ParTemValue();
                    parTemValue3.setSetTemValue(setUpTemp.floatValue());
                    parTemValue3.setActTemValue(actualTemp.floatValue());
                    parTemValue3.setOuterZone1Temp(outerZoneITemperature.floatValue());
                    parTemValue3.setOuterZone2Temp(outerZone2Temperature.floatValue());
                    parTemValue3.setFurnacePressure(furnacePressure.floatValue());
                    parTemValue3.setKilnId(kilnId);
                    parTemValue3.setTaskId(currTaskId);
                    setCommonField(parTemValue3);
                    parTemValueServiceBiz.save(parTemValue3);
                    //电度值
                    Double electricalValue = readPlc(nameSpaceNote,"ElectricalValue");
                    //A向电压
                    Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                    Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                    Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                    ParEleValue parEleValue3=new ParEleValue();
                    parEleValue3.setAVoltagevValue(phaseAVoltage.floatValue());
                    parEleValue3.setAVoltagevValue(phaseBVoltage.floatValue());
                    parEleValue3.setAVoltagevValue(phaseCVoltage.floatValue());
                    parEleValue3.setTotalBat(electricalValue.floatValue());
                    parEleValue3.setKilnId(kilnId);
                    parEleValue3.setTaskId(currTaskId);
                    setCommonField(parEleValue3);
                    parEleValueServiceBiz.save(parEleValue3);
                }
                //清洗炉
                if (kilnInfo.getType()==4){
                    //电度值
                    Double electricalValue = readPlc(nameSpaceNote,"TotalElectricity");
                    //A向电压
                    Double phaseAVoltage = readPlc(nameSpaceNote,"PhaseAVoltage");
                    Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                    Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                    ParEleValue parEleValue4=new ParEleValue();
                    parEleValue4.setAVoltagevValue(phaseAVoltage.floatValue());
                    parEleValue4.setAVoltagevValue(phaseBVoltage.floatValue());
                    parEleValue4.setAVoltagevValue(phaseCVoltage.floatValue());
                    parEleValue4.setTotalBat(electricalValue.floatValue());
                    parEleValue4.setKilnId(kilnId);
                    parEleValue4.setTaskId(currTaskId);
                    setCommonField(parEleValue4);
                    parEleValueServiceBiz.save(parEleValue4);
                }
            }
        }
    }

    //遍历窑炉，空窑炉查询缓存区是否有等待加工的任务，
    //待加工任务存在且没有状态为正在进炉的情况下，按照先进先出的规则选择待加工任务呼叫车辆进炉
    public void runTaskForEmptyKiln() throws IOException {
        List<KilnInfo> kilnInfoList = kilnInfoServiceBiz.list(new QueryWrapper<KilnInfo>()
                .eq(KilnInfo.VALID, 1).eq(KilnInfo.STATUS, 0));
        //存在空闲窑炉
        if (kilnInfoList.size()!=0){
            //缓存去是否有目标位置为窑炉的待加工任务
            KilnInfo kilnInfo = kilnInfoList.get(0);
            List<InStockInfo> inStockInfoList = inStockInfoServiceBiz.list(new QueryWrapper<InStockInfo>()
                    .eq(InStockInfo.TYPE, 1)
                    .eq(InStockInfo.KILN_ID, kilnInfo.getId())
                    .orderByAsc(InStockInfo.IN_TIME));
            if (inStockInfoList.size()>0){
                InStockInfo inStockInfo = inStockInfoList.get(0);
                CurrTask currTask = currTaskServiceBiz.getById(inStockInfo.getTaskId());
                Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
                //是否有小车在运行
                List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>().eq(VehicleInfo.STATUS, 1));
                //两辆车同时空闲（其中一辆车在运行时，不能调度另一辆车，防止运行中路线冲突）
                if (vehicleInfoList.size()==0){
                    //计算出路径最短的车辆id
                    Long vehicleId = taskDistanceUtils.chooseVehicle(location.getCode(), kilnInfo.getCode());
                    //占用车辆
                    VehicleInfo vehicleInfo = vehicleInfoServiceBiz.getById(vehicleId);
                    vehicleInfo.setStatus(1);
                    vehicleInfo.setUpdateTime(LocalDateTime.now());
                    vehicleInfoServiceBiz.updateById(vehicleInfo);
                    asynRunTaskService.asynRunTaskForStockToKiln(inStockInfo.getId(),vehicleId);
                }
            }

        }
    }

    //遍历正在工作炉子的报警变量，记录报警
    public void listenKilnAlarm(){
        List<KilnInfo> kilnInfoList = kilnInfoServiceBiz.list();
        for (KilnInfo kilnInfo:kilnInfoList
             ) {
            PlcNameSpace kilnNameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                    .eq(PlcNameSpace.EQ_TYPE, 0)
                    .eq(PlcNameSpace.EQ_ID, kilnInfo.getId()));
            //判断炉子是否在工作状态
            String working = readPlcToString(kilnNameSpace.getName(), "Working");
            JSONObject jsonObject = JSON.parseObject(working);
            Boolean aBoolean = Boolean.valueOf(String.valueOf(jsonObject.get("msg")));
            if (aBoolean){
                //命名空间前缀
                String nameSpace = kilnNameSpace.getNote();
                //炉子的报警变量
                List<PlcNameSpace> alarmNameList = plcNameSpaceServiceBiz.list(new QueryWrapper<PlcNameSpace>()
                        .eq(PlcNameSpace.EQ_TYPE, 1)
                        .eq(PlcNameSpace.TYPE, 1)
                        .eq(PlcNameSpace.EQ_ID,kilnInfo.getId()));;
                for (PlcNameSpace plcNameSpace:alarmNameList
                ) {
                    String name = plcNameSpace.getName();
                    Map<String, Object> json = new HashMap();
                    json.put("nameSpace", 6);
                    json.put("plcName", "plc1");
                    json.put("identifier",nameSpace.concat(name));
                    String jsonString = JSON.toJSONString(json);
                    try {
                        String result = HttpClient.httpPost("http://localhost:8009/opcua/read", jsonString);
                        Boolean resultBoolean = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
                        //报警状态为true
                        if (resultBoolean){
                            //任务状态为执行中且正在炉子中加工的任务
                            CurrTask currTask = currTaskServiceBiz.getOne(new QueryWrapper<CurrTask>()
                                    .eq(CurrTask.KILN_ID, kilnInfo.getId())
                                    .eq(CurrTask.IS_IN, 1)
                                    .eq(CurrTask.STATUS, 1));
                            AlarmInfo alarmInfo=new AlarmInfo();
                            alarmInfo.setEquipmentId(currTask.getKilnId());
                            alarmInfo.setEquipmentName(currTask.getKilnName());
                            alarmInfo.setType(0);
                            alarmInfo.setAlarmCode(CodeGeneratorHelper.getAlarmCode());
                            alarmInfo.setAlarmInfo(plcNameSpace.getEqName()+"："+plcNameSpace.getNote());
                            alarmInfo.setTaskCode(currTask.getTaskCode());
                            alarmInfo.setCreateTime(LocalDateTime.now());
                            alarmInfoServiceBiz.save(alarmInfo);
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

    }

    /**
     * 获取变量值
     * @param nameSpace nameSpace前缀
     * @param variableName 变量名称
     * @return result字符串
     */
    private String readPlcToString(String nameSpace, String variableName){
        String identifier=nameSpace.concat(variableName);
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier",identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://localhost:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("readPLC： "+identifier+" 的值失败");
        }
        return result;
    }
    /**
     * 获取变量值
     * @param nameSpace nameSpace前缀
     * @param variableName 变量名称
     * @return 变量值
     */
    private Double readPlc(String nameSpace, String variableName){
        String identifier=nameSpace.concat(variableName);
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier",identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://localhost:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("能源消耗统计：获取 "+identifier+" 的值失败");
        }
        return Double.valueOf(JSONObject.parseObject(result).get("result").toString());
    }
}
